/*
 * funcs.h
 *
 * rcsid = "$Header: /nfs/globus1/src/master/nexus/funcs.h,v 1.91 1998/02/25 20:33:12 toonen Exp $"
 */

#ifndef _NEXUS_INCLUDE_FUNCS_H
#define _NEXUS_INCLUDE_FUNCS_H

/*
 * args.c
 */
void
_nx_get_args(int *argc,
	     char ***argv,
	     char *args_env_variable,
	     char *package_designator);

extern void
_nx_args_init(int *argc, char ***argv);

extern void
_nx_args_cleanup(int *argc, char ***argv);

extern int
_nx_split_args(char *arg_string,
	       int starting_argc,
	       int max_argc,
	       char **package_argv,
	       char *string_buffer);

extern char *
_nx_get_package_id_start(void);

extern char *
_nx_get_package_id_end(void);

extern char *
_nx_get_argv0(void);

/*
 * attach.c
 */
extern void	_nx_attach_usage_message(void);
extern int	_nx_attach_new_process_params(char *buf, int size);
extern void	_nx_attach_init(int *argc, char ***argv);


/*
 * buffer.c
 */
extern void _nx_buffer_init(int *argc, char ***argv);
extern int  _nx_buffer_create_from_raw(nexus_byte_t *raw_buffer,
				       unsigned long raw_size,
				       unsigned long offset_to_message_start,
				       unsigned long message_length,
				       nexus_endpoint_t *endpoint,
				       struct _nexus_buffer_t **buffer);
extern int  _nx_buffer_dispatch(struct _nexus_buffer_t *buffer);


/*
 * commlink.c
 */
extern void	_nx_commlink_usage_message(void);
extern void	_nx_commlink_init(int *argc, char ***argv);
extern int	_nx_commlink_new_process_params(char *buf, int size);
extern int	_nx_write_startpoint(int filedesc, nexus_startpoint_t *sp);
extern int	_nx_read_startpoint(int filedesc, nexus_startpoint_t *sp);


/*
 * context.c
 */
extern void	_nx_context_usage_message(void);
extern void	_nx_context_init(int *argc, char ***argv);
extern void	_nx_context_endpointattr_init();
extern nexus_context_t *_nx_context_alloc();
extern int	_nx_context_new_process_params(char *buf, int size);


/*
 * fault.c
 */
extern void	_nx_fault_tolerance_usage_message(void);
extern int	_nx_fault_tolerance_new_process_params(char *buf, int size);
extern void	_nx_fault_tolerance_init(int *argc, char ***argv);
extern int	_nx_fault_detected(int fault_code);


/*
 * md_*.c
 */
extern void	_nx_md_usage_message(void);
extern void	_nx_md_init(int *argc, char ***argv);
extern void	_nx_md_shutdown(void);
extern void	_nx_md_abort(int return_code);
extern void	_nx_md_exit(int return_code);
extern int	_nx_md_get_command_line_args(int starting_argc,
					     int max_argc,
					     char **argv);
extern char *	_nx_md_system_error_string(int the_error);
extern char *	_nx_md_get_unique_session_string();
extern void	_nx_md_gethostname(char *name, int len);
extern int	_nx_md_getpid(void);
extern int	_nx_md_new_process_params(char *buf, int size);
extern int	_nx_md_fork(void);


/*
 * pr_iface.c
 */
extern void	_nx_proto_usage_message(void);
extern int	_nx_proto_new_process_params(char *buf, int size);
extern void	_nx_proto_init(int *argc,
			       char ***argv,
			       nexus_module_list_t module_list[]);
extern void	_nx_proto_shutdown(nexus_bool_t shutdown_others);
extern void	_nx_proto_abort(void);
extern void	_nx_proto_get_creator_sp_params(char *buf,
						int buf_size,
						nexus_startpoint_t *sp);
extern void	_nx_proto_construct_creator_sp(nexus_startpoint_t *creator_sp);
extern void	_nx_mi_proto_destroy(nexus_mi_proto_t *mi_proto);
extern nexus_mi_proto_t *_nx_mi_proto_table_insert(
					nexus_mi_proto_t *new_mi_proto);
extern nexus_mi_proto_t *_nx_mi_proto_create(int size,
					     nexus_byte_t *array,
					     nexus_proto_t *proto);
extern nexus_mi_proto_t *_nx_mi_proto_create_for_proto_type(
						nexus_proto_type_t type,
						void *proto_info,
						nexus_endpoint_t *endpoint);
extern int _nx_mi_proto_destroy_for_proto_type(nexus_endpoint_t * endpoint,
					       nexus_mi_proto_t *mi_proto);


/*
 * pr_local.c
 */
extern void *	_nx_pr_local_info(void);


#ifdef BUILD_RESOURCE

/*
 * pr_iface.c
 */

nexus_resource_name_t * 
_nx_mi_proto_name_for_proto_type( nexus_proto_type_t proto_type,
				  nexus_endpoint_t *endpoint );



/*
 * overseer_iface.c
 */

extern void nexus_overseer_init(int *argc, 
				char ***argv,
				nexus_module_list_t module_list[]);

#endif /* BUILD_RESOURCE */


/*
 * rdb_iface.c
 */
extern void	_nx_rdb_usage_message(void);
extern int	_nx_rdb_new_process_params(char *buf, int size);
extern void	_nx_rdb_init(int *argc,
			     char ***argv,
			     nexus_module_list_t module_list[]);
extern void	_nx_rdb_shutdown(void);
extern void	_nx_rdb_abort(void);
extern nexus_bool_t _nx_rdb_hash_table_lookup(char *name,
					      char *key,
					      char **value);
extern void     _nx_rdb_hash_table_add(char *name,
				       nexus_list_t *attr);
extern nexus_list_t *_nx_rdb_parse_attributes(char *buf,
					      int buf_len,
					      char *starting_point,
					      FILE *fp);
extern void     _nx_rdb_flush_rest_of_line(char *buf,
					   int buf_len,
					   char *starting_point,
					   FILE *fp);
extern void     _nx_rdb_hash_table_add_nodes_with_attrs(
			nexus_rdb_hash_entry_t *nodes,
			nexus_list_t *attrs);

/*
 * th_*.c
 */
#ifndef BUILD_LITE
extern void	_nx_thread_usage_message(void);
extern int	_nx_thread_new_process_params(char *buf, int size);
extern void	_nx_thread_preinit(void);
extern void	_nx_thread_init(int *argc, char ***argv);
extern void	_nx_thread_shutdown(void);
extern void	_nx_thread_abort(int return_code);
extern void	_nx_thread_create_handler_thread(void);
extern void	_nx_thread_shutdown_handler_thread(void);
#endif /* BUILD_LITE */


/*
 * trace.c
 */
extern void	_nx_traceback(void);


/*
 * transform_iface.c
 */
extern void	nexus_transform_usage_message(void);
extern int	nexus_transform_new_process_params(char *buf, int size);
extern int	nexus_transform_init(int *argc, char ***argv,
				     nexus_module_list_t module_list[]);
extern void	nexus_transform_add_module(char *module_name,
					   void *(*info_func)(void),
					   int *argc, char ***argv);
extern void     nexus_transform_shutdown(void);
extern void     nexus_transform_abort(void);
extern void	nexus_transformattr_init(int id,
					 void *info,
					 nexus_transformattr_t **attr);
extern void	nexus_transformattr_destroy(int id,
					    nexus_transformattr_t *attr);
extern void	nexus_transformattr_get_info(int id,
					     nexus_transformattr_t *attr,
					     void **info);
extern void	nexus_transformstate_init_on_endpoint(int id,
					   nexus_transformattr_t *attr,
					   nexus_transformstate_t **ep_state);
extern void	nexus_transformstate_destroy_on_endpoint(int id,
					nexus_transformstate_t *ep_state);
extern void	nexus_transformstate_update_endpoint_with_startpoint_destroy(
				int id,
				nexus_transformstate_t *ep_state,
				nexus_transformstate_t *sp_state);
extern void	nexus_transformstate_init_on_startpoint(int id,
					     nexus_transformstate_t *ep_state,
					     nexus_transformstate_t **sp_state,
					     nexus_bool_t *copy_sp_locally,
					     nexus_bool_t *destroy_sp_locally);
extern void	nexus_transformstate_copy(int id,
			       nexus_transformstate_t *sp_state,
			       nexus_transformstate_t **sp_state_copy);
extern void	nexus_transformstate_destroy_on_startpoint(
					int id,
    					nexus_transformstate_t *sp_state);
extern int	nexus_transformstate_sizeof(int id,
					    nexus_transformstate_t *sp_state);
extern void	nexus_transformstate_put(int id,
					 nexus_byte_t **buffer,
					 nexus_transformstate_t *sp_state);
extern void	nexus_transformstate_get(int id,
					 nexus_byte_t **buffer,
					 int format,
					 nexus_transformstate_t **sp_state);
extern int	nexus_buffer_transform(nexus_startpoint_t *sp,
				       nexus_byte_t *storage_start,
				       unsigned long storage_size,
				       nexus_byte_t *data_start,
				       unsigned long *data_size,
				       nexus_bool_t must_alloc_new_buffer,
				       nexus_byte_t *transform_info_start,
				       nexus_byte_t **out_storage_start,
				       unsigned long *out_storage_size,
				       nexus_byte_t **out_data_start,
				       unsigned long *out_data_size);
extern int	nexus_buffer_untransform(nexus_endpoint_t *ep,
					 nexus_byte_t *data_start,
					 unsigned long *data_size,
					 nexus_byte_t *transform_info_start,
					 int format,
					 nexus_byte_t **destination_start,
					 unsigned long *destination_size);
extern void	nexus_transform_info(int transform_id,
				     nexus_bool_t *modifies_data,
				     unsigned long *transform_info_size,
				     unsigned long *trailer_size);

/*
 * util.c
 */
extern char *	_nx_copy_string(char *s);
extern char *	_nx_executable_name(void);
extern char *	_nx_current_working_directory(void);
extern void	_nx_strip_tmp_mnt_from_path(char *dir);
extern char *   _nx_find_attribute(char *attr,
				   char *search_string,
				   char separator);
extern void     _nx_get_next_value(char *values,
				   char separator,
				   char **next,
				   char **this);
extern void	_nx_hex_encode_byte_array(unsigned char *bytes,
					  int length,
					  char *hex);
extern void	_nx_hex_decode_byte_array(char *hex,
					  int length,
					  unsigned char *bytes);
#ifndef HAVE_STRTOUL
extern unsigned long strtoul(const char *nptr,
			     char **endptr,
			     int base);
#endif /* HAVE_STRTOUL */
extern int	_nx_write_blocking(int fd, void *buf, size_t size);
extern int	_nx_read_blocking(int fd, void *buf, size_t size);

#endif /* _NEXUS_INCLUDE_FUNCS_H */

