/*
 * md_unix.c
 */

static char *rcsid = "/net/bcomp/src/master/nexus/md_unix.c,v 1.51 1995/09/15 16:36:57 tuecke Exp";

#include "internal.h"

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <sys/time.h>

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>


/*
 * _nx_md_usage_message()
 */
void _nx_md_usage_message(void)
{
} /* _nx_md_usage_message() */


/*
 * _nx_md_new_process_params()
 */
int _nx_md_new_process_params(char *buf, int size)
{
    return(0);
} /* _nx_md_new_process_params() */


/*
 * _nx_md_init()
 *
 * Initialize any machine dependent stuff
 */
void _nx_md_init(int *argc, char ***argv)
{
} /* _nx_md_init() */


/*
 * _nx_md_shutdown()
 *
 * Shutdown any machine dependent stuff.
 */
void _nx_md_shutdown(void)
{
} /* _nx_md_shutdown() */


/*
 * _nx_md_abort()
 */
void _nx_md_abort(int return_code)
{
    exit(return_code);
} /* _nx_md_abort() */


/*
 * _nx_md_exit()
 */
void _nx_md_exit(int return_code)
{
    exit(return_code);
} /* _nx_md_exit() */


/*
 * _nx_md_get_command_line_args()
 *
 * Retrieve the command line arguments and fill them into 'argv',
 * starting at 'starting_argc', up to a maximum of 'max_argc'.
 * Return a new argc.  The char* pointers of argv[] will be set
 * to strings in malloced memory.
 *
 * There is a chicken and egg problem.  With CC++, the global
 * constructors run before main() is called with argc/argv.
 * Thus, Nexus needs to be setup by the first global constructor,
 * and that Nexus setup may need command line arguments to configure
 * itself properly.  However, since main() as not yet been called
 * we do not yet have argc/argv.
 *
 * Fortunately, most versions of Unix store the command line
 * arguments in the negative environ[] locations, so they can
 * actually be retrieved before main() is even called.  This is
 * hack used here.
 *
 * If argc/argv are not retrievable prior to main() being called,
 * then the user must resort to using the package environment
 * variable to set configuration options.
 *
 * Note: If need be, _nx_split_args() can be called to split
 * a string into its component arguments.
 *
 * Return: A new argc, which is the element in argv immediately
 *		after the last element that we put into it.
 */
int _nx_md_get_command_line_args(int starting_argc,
				 int max_argc,
				 char **argv)
{

    /* This is not supported on the following machines:
     *		IBM SP, POE programming, AIX4
     */

#if defined(TARGET_ARCH_AIX4) && defined(HAVE_MPL)
    return(0);

#elif defined(TARGET_ARCH_CRAYC90)
    
    /*
     * On the Cray,  the arguments are stored in reverse order
     * in environ[] from -2 to -base_arg, where environ[-(base_arg-1)]
     * is the program name.
     *
     * So structure of environ (for most machines) seems to be:
     *
     * -(argc+4)      -(argc+1)              -3       -2       -1       0
     *     argc |.|.| argv[argc-1] | ... | argv[1] | argv[0] | NULL | envars
     *             :
     *         -base_arg
     *
     * base_arg can be found by scanning back through environ[]
     * until (environ[-(base_arg+2)]==base_arg-2) (that is, until
     * we find argc in environ[]).
     */
    
    int base_arg;
    int argc, current_argc;
    int i, arg;
    extern char **environ;

    /*
     * Find the first argument in environ[] and initialize
     * base_arg with that value.
     */
    for (base_arg = 2; 
	 (int) environ[-(base_arg+2)] != base_arg-2 && 
	 environ[-base_arg] != (char *) 0; 
	 base_arg++
	 )
	;
    argc = base_arg - 2;

    if (starting_argc + argc > max_argc)
    {
	printf("_nx_md_get_command_line_args(): Error: Too many command line arguments\n");
	_nx_md_exit(1);
    }

    current_argc = starting_argc;
    for (i = 0; i < argc; i++)
    {
	arg = base_arg - i - 1;
	argv[current_argc++] = _nx_copy_string(environ[-arg]);
    }

    return (current_argc);
    
#elif 0 /* defined(TARGET_ARCH_PARAGON) */
	/* Well hack doesn't work on the Paragon so need to use
	Environment variable */
	int new_argc;
	int current_argc = starting_argc;
	char* nexus_args_str = getenv(NEXUS_ARGS);
	char** env;
	extern char** environ;

	env = environ;
#if defined (BUILD_DEBUG) && defined (TAL)
	if (NexusDebug (2)) {
	if (mynode() > 0) {
	printf ("In _nx_md_get_command_line_args() Environ: .\n");
	while (*env != NULL) {
		nexus_printf ("%s\n", *env++);
	}
	}
	nexus_printf ("In _nx_md_get_command_line_args() -- PGON.\n");
	nexus_printf ("nexus_args_str: %s\n", nexus_args_str);
	}
#endif
	if (nexus_args_str) {
		if ((new_argc =
		 _nx_split_args (nexus_args_str, starting_argc, 
		 max_argc, argv, NULL)) < 0) {
			nexus_printf (
			 "Error while reading args from Environment %s\n", NEXUS_ARGS);
			_nx_md_exit(1);
		}
		current_argc = new_argc;
	}
	else {
		nexus_printf (
		 "Nexus on Paragon requires placing runtime flags in Environment %s\n",
		 NEXUS_ARGS);
		_nx_md_exit(1);
	}

#if defined (BUILD_DEBUG) && defined (TAL)
	if (NexusDebug (2)) {
	nexus_printf ("In _nx_md_get_command_line_args().\n");
	nexus_printf ("%d current_argc\n", current_argc);
	for (new_argc = 0; new_argc < current_argc; new_argc++) 
		nexus_printf ("--argv[%d]: %s\n", new_argc, argv[new_argc]);
	}
#endif
	return (current_argc);

#else
    
    /*
     * The structure of environ (for most machines) seems to be:
     *
     * -(argc+2)  -(argc+1)  -argc                -2       -1       0
     *     argc | argv[0] | argv[1] | ... | argv[argc-1] | NULL | envars
     * -base_arg
     * 
     * The RS/6000 puts a NULL in environ[-(argc+2)] instead of argc.
     *
     * This has been tested on at least the following machines:
     *   Sun (Sparc) SunOS 4.x, Sun (Sparc) Solarix 2.x,
     *   IBM RS/6000 AIX, SGI Indigo, Sequent Symmetry Dynix,
     *   Intel iPSC/860 SRM (386 running SysV 3.2),
     *   NEXTSTEP (486 and Motorola), PC running SysVr4
     */

#ifdef TARGET_ARCH_PARAGON    
#define ArgStart 2
    int null_pos = ArgStart;
#endif    
    int base_arg;
    int argc, current_argc;
    int i, arg;
    extern char **environ;

    /*
     * Find the first argument in environ[] and initialize
     * base_arg with that value.
     */
#ifndef TARGET_ARCH_PARAGON
    
    for (base_arg = 2; 
	 (long) environ[-(base_arg)] != base_arg-2 && 
	 environ[-base_arg] != (char *) 0; 
	 base_arg++
	 )
	;
    argc = base_arg - 2;
    if (starting_argc + argc > max_argc)
    {
	printf("_nx_md_get_command_line_args(): Error: Too many command line arguments\n");
	_nx_md_exit(1);
    }

    current_argc = starting_argc;
    for (i = 0; i < argc; i++)
    {
	arg = base_arg - i - 1;
	argv[current_argc++] = _nx_copy_string(environ[-arg]);
    }
    
#else /* TARGET_ARCH_PARAGON */
    
    for (base_arg = null_pos;
	 (int) environ [-base_arg] != base_arg-ArgStart;
	 base_arg++)
    {
	if (environ [-base_arg] == NULL)
	    null_pos = base_arg;
    }
	
    argc = base_arg - null_pos -1;
    nexus_debug_printf(2,("argc %d, base_arg %d, null_pos %d.\n", argc, base_arg, null_pos));

    if (starting_argc + argc > max_argc)
    {
	printf("_nx_md_get_command_line_args(): Error: Too many command line arguments\n");
	_nx_md_exit(1);
    }

    current_argc = starting_argc;
    for (i = 0, base_arg--; i < argc; i++, base_arg--)
    {
	/*
	arg = base_arg - i - 1;
	*/
	/*
	printf("(%ld,%ld) current_argc %d env[-%d] %s\n", mynode(), myptype(),
	       current_argc, base_arg, environ[-base_arg]);
	*/
	argv[current_argc++] = _nx_copy_string(environ[-base_arg]);
    }
    
#endif /* TARGET_ARCH_PARAGON */


    return (current_argc);
    
#endif /* TARGET_ARCH_CRAYC90 */
    
} /* _nx_md_get_command_line_args() */


/*
 * _nx_md_system_error_string()
 *
 * Return the string for the current errno.
 */
char *_nx_md_system_error_string(int the_error)
{
#ifndef TARGET_ARCH_LINUX
    extern char *sys_errlist[];
#endif
    return ((char *)sys_errlist[the_error]);
} /* _nx_md_system_error_string() */


/*
 * _nx_md_get_unique_session_string()
 *
 * Return a malloced string containing a unique string.
 * This string should be unique for all time, not just within
 * this process but across all process on all machines.
 * This string is composed of my hostname, process id, and the
 * current time (seconds since 1970).
 */
char *_nx_md_get_unique_session_string()
{
    char hostname[MAXHOSTNAMELEN];
    char tmp_buf[MAXHOSTNAMELEN + 32];
    _nx_md_gethostname(hostname, MAXHOSTNAMELEN);
    nexus_stdio_lock();
    sprintf(tmp_buf, "%s_%lx_%lx",
	    hostname,
	    (unsigned long) _nx_md_getpid(),
	    (unsigned long) time(0));
    nexus_stdio_unlock();
    if (strlen(tmp_buf) >= NEXUS_MAX_SESSION_STRING_LENGTH)
    {
	nexus_fatal("Internal Error: NEXUS_MAX_SESSION_STRING_LENGTH=%d not long enough to hold seesion string\n", NEXUS_MAX_SESSION_STRING_LENGTH);
    }
    return(_nx_copy_string(tmp_buf));
} /* _nx_md_get_unique_session_string() */

    
/*
 * _nx_md_gethostname()
 *
 * Return the hostname that this process is running on.
 * Do any error checking.
 *
 * Lookup the "domain" for this hostname in the resource database.
 * If it is present, then add it to this hostname if the hostname
 * doesn't already end in that domain.
 */
void _nx_md_gethostname(char *name, int len)
{
    static char hostname[MAXHOSTNAMELEN];
    static int hostname_length = -1;

    if (hostname_length == -1)
    {
	struct hostent *	hp_ptr;
#if defined(TARGET_ARCH_LINUX) && defined(HAVE_PTHREAD)
	struct hostent hp2;
	char hp_tsdbuffer[500];
	int hp_errnop;
	int rc;
#elif HAVE_GETHOSTBYNAME_R_5
	struct hostent hp2;
	char hp_tsdbuffer[500];
	int hp_errnop;
#elif HAVE_GETHOSTBYNAME_R_3
	struct hostent hp2;
	struct hostent_data hp_data;
	int rc;
#endif

	if (gethostname(hostname, MAXHOSTNAMELEN) < 0)
	{
	    nexus_fatal("_nx_md_gethostname(): gethostname() failed\n");
	}
	hostname_length = strlen(hostname);

#if defined(TARGET_ARCH_LINUX) && defined(HAVE_PTHREAD)
	printf ("hostname(%d): %s\n", hostname_length, hostname);
	rc = gethostbyname_r(hostname, &hp2, hp_tsdbuffer, 500, 
			     &hp_ptr, &hp_errnop);
	if (rc != 0) {
	  hp_ptr = NULL;
	}
	nexus_stdio_lock();
#elif defined(HAVE_GETHOSTBYNAME_R_5)
	hp_ptr = gethostbyname_r(hostname, &hp2, hp_tsdbuffer, 500, &hp_errnop);
	nexus_stdio_lock();
#elif defined(HAVE_GET_HOSTBYNAME_R_3)
	rc = gethostbyname_r(hostname, &hp2, &hp_data);
	if (rc == 0) {
	  hp_ptr = &hp2;
	}
	else {
	  hp_ptr = NULL;
	}
	nexus_stdio_lock();
#else
	/*
	 * On SunOS 4.1.X, gethostbyname() calls sprintf() :(
	 */
	nexus_stdio_lock();
	hp_ptr = gethostbyname(hostname);
#endif

	if (hp_ptr != NULL)
	{
	    struct in_addr	addr;

#ifdef TARGET_ARCH_CRAYT3E
            memcpy(&(addr.s_da), hp_ptr->h_addr, hp_ptr->h_length);
            hp_ptr = gethostbyaddr((const char *)&addr,sizeof(addr),AF_INET);
#else
	    memcpy(&(addr.s_addr), hp_ptr->h_addr, hp_ptr->h_length);
	    hp_ptr = gethostbyaddr((void *) &addr,sizeof(addr),AF_INET);
#endif
	    if (hp_ptr != NULL && strcmp(hp_ptr->h_name, hostname) != 0)
	    {
		strcpy(hostname, hp_ptr->h_name);
	    }
	    else
	    {
		hp_ptr = NULL;
	    }
	}
	nexus_stdio_unlock();

	if (hp_ptr == NULL)
	{
	    char *		domain;
	    int			domain_length;
	
	    domain = nexus_rdb_lookup(hostname, "domain");
	    if (domain)
	    {
		domain_length = strlen(domain);
		if ((domain_length >= hostname_length)
		    || strcmp(&(hostname[hostname_length-domain_length]),
			      domain) != 0)
		{
		    if (hostname_length + domain_length >= MAXHOSTNAMELEN)
		    {
			nexus_fatal("_nx_md_gethostname(): "
				    "hostname+domain is too long\n");
		    }
		    strcat(hostname, domain);
		    hostname_length += domain_length;
		}

		nexus_rdb_free(domain);
	    }
	}
    }

    if (hostname_length < len)
    {
	strcpy(name, hostname);
    }
    else
    {
	nexus_fatal("_nx_md_gethostname(): hostname too long\n");
    }
} /* _nx_md_gethostname() */


/*
 * _nx_md_getpid()
 */
int _nx_md_getpid(void)
{
    static int pid = -1;
    if (pid == -1)
    {
	pid = (int) getpid();
    }
    return(pid);
} /* _nx_md_getpid() */


/*
 * _nx_md_fork()
 *
 * Fork a child process with just one thread.
 * Register this child so that we can wait for it later.
 */
int _nx_md_fork(void)
{
    int child;

#ifndef BUILD_LITE
    _nx_thread_prefork();
#endif /* BUILD_LITE */
    
#ifdef HAVE_FORK1
    child = fork1();
#else
    child = fork();
#endif
    
#ifndef BUILD_LITE
    if (child > 0)
    {
	_nx_thread_postfork();
    }
#endif /* BUILD_LITE */
    
    return(child);
} /* _nx_md_fork() */


/*
 * nexus_usleep()
 *
 * Sleep for usleep microseconds.
 */
void nexus_usleep(long usec)
{
    struct timeval timeout;
    
    timeout.tv_sec = usec/1000000;
    timeout.tv_usec = usec%1000000;
    
    select(0, NULL, NULL, NULL, &timeout);
} /* nexus_usleep() */
     

/*
 * nexus_wallclock()
 *
 * Return the current wallclock time in seconds.
 */
double nexus_wallclock(void)
{
    struct timeval tv;

    gettimeofday(&tv, 0);
    return (((double) tv.tv_sec) + ((double) tv.tv_usec) / 1000000.0);
} /* nexus_wallclock() */


/*
 * nexus_pause()
 */
void nexus_pause(void)
{
    nexus_printf("Process %d pausing...\n", _nx_md_getpid());
    pause();
} /* nexus_pause() */
