/*
 * ep_list.c
 */

#include "ep_list.h"
#include "harness_options.h"

typedef struct
{
    nexus_endpoint_t	*endpoint;
    void		(*cleanup)(nexus_endpoint_t *ep);
    int			free_ep;
} ep_info_t;

static ep_info_t	ep_list[MAX_N_ENDPOINTS];
static int		next_ep;

static int
unique_ep_id()
{
    int		e;

    e = next_ep++;
    if (e >= MAX_N_ENDPOINTS) {
	nexus_printf("ran out of endpoints!\n");
	next_ep--;
	return (-1);
    }
    return (e);
}

int
ep_list_startup()
{
    int	i;
    
    HelloEpl(nexus_printf("ep_list_startup(): hello\n"));
    next_ep = 0;
    for (i = 0; i < MAX_N_ENDPOINTS; i++) {
	ep_list[i].endpoint = NULL;
	ep_list[i].cleanup = NULL;
    }
    return (0);
}

void
ep_list_shutdown(int aborted)
{
    int		i;

    HelloEpl(nexus_printf("ep_list_shutdown(): hello\n"));
    for (i = 0; i < MAX_N_ENDPOINTS; i++)
	if (ep_list[i].endpoint != NULL) {
	    if (aborted == NEXUS_FALSE)
		nexus_printf("warning: ep_list[%i] != NULL\n", i);
	    ep_list_destroy_ep(i);
	}
}

int
ep_list_new_ep(nexus_endpoint_t *ep,
	       nexus_bool_t free_ep,
	       void (*cleanup_func)(nexus_endpoint_t *))
{
    int r = unique_ep_id();

    if (ep == NULL) {
	nexus_printf("ep_list_new_ep(): ep = NULL\n");
	return (1);
    }
    TraceEpl(nexus_printf("Adding endpoint[%i]\n", r));
    ep_list[r].endpoint = ep;
    ep_list[r].free_ep = free_ep;
    ep_list[r].cleanup = cleanup_func;
    return (r);
}

nexus_endpoint_t *
ep_list_get_ep(int ep_id)
{
    if ((ep_id >= 0) && (ep_id < next_ep))
	return (ep_list[ep_id].endpoint);
    else {
	nexus_printf("FATAL ERROR: ep_list_get_ep(%i)\n", ep_id);
	nexus_abort();
	return (NULL); /* don't get here */
    }
}

void
ep_list_destroy_ep(int ep_id)
{
    if ((ep_id < 0) || (ep_id >= next_ep) ||
	(ep_list[ep_id].endpoint == NULL)) {
	nexus_printf("FATAL ERROR: ep_list_delete(%i)\n", ep_id);
	nexus_abort();
    } else {
	if (ep_list[ep_id].cleanup != NULL)
	    (ep_list[ep_id].cleanup)(ep_list[ep_id].endpoint);
	nexus_endpoint_destroy(ep_list[ep_id].endpoint);
	if (ep_list[ep_id].free_ep != NEXUS_FALSE)
	    nexus_free(ep_list[ep_id].endpoint);
	ep_list[ep_id].endpoint = NULL;
	ep_list[ep_id].cleanup = NULL;
	ep_list[ep_id].free_ep = NEXUS_FALSE;
    }
}
