/******************************************************************************
CIF_Connection_Nexus.H

Description:

  Connection class for Nexus based protocols

CVS Information:

  $Source$
  $Date$
  $Revision$
  $State$
  $Author$
******************************************************************************/

#if !defined(CIF_INCLUDE_CIF_CONNECTION_NEXUS)
#define CIF_INCLUDE_CIF_CONNECTION_NEXUS 1

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_Connection.H"
#include "Nexus_internal.H"


/******************************************************************************
		       Define CIF_Connection_Nexus class
******************************************************************************/

class
CIF_Connection_Nexus
: public CIF_Connection
{
    /**************************************************************************
				    API methods
    **************************************************************************/

    /*
     * CIF_Connection_Nexus::CIF_Connection_Nexus()
     *
     */
    public:
    CIF_Connection_Nexus();

    /*
     * CIF_Connection_Nexus::open()
     */
    public: virtual
    CIF_Bool
    open(
	ARG_IN char *			url,
	ARG_IN int			timeout = -1);

    /*
     * CIF_Connection_Nexus::open()
     */
    public: virtual
    CIF_Bool
    open(
	ARG_INMOD Nexus_Endpoint &	local_ctrl_endpoint,
	ARG_IN int			timeout = -1);

    /*
     * CIF_Connection_Nexus::send()
     */
    public: virtual
    CIF_Bool
    send(
	ARG_IN CIF_Byte *		msg_buf,
	ARG_IN CIF_Size			msg_len,
	ARG_IN int			msg_tag);
    
    /*
     * CIF_Connection_Nexus::receive()
     */
    public: virtual
    CIF_Bool
    receive(
	ARG_OUT CIF_Message &		message,
	ARG_IN int			timeout = -1);

    /*
     * CIF_Connection_Nexus::poll()
     */
    public: virtual
    CIF_Bool
    poll();

    /*
     * CIF_Connection_Nexus::close()
     *
     * Close an established or faulty connection
     */
    public: virtual
    CIF_Bool
    close();

    /*
     * CIF_Connection_Nexus::~CIF_Connection_Nexus()
     *
     * Free any resources used by the connection.  If the connection is still
     * open, it will be closed.
     *
     * NOTE: all queued messages will be discarded and their resources freed
     */
    public: virtual
    ~CIF_Connection_Nexus();

    
    /**************************************************************************
	        Methods and data accessible only by derived classes
    **************************************************************************/

    /*
     * CIF_Connection_Nexus::establish_Channel()
     */
    protected: virtual
    CIF_Bool
    establish_Connection(
	ARG_IN char *			url,
	ARG_OUT Nexus_Endpoint &	local_comm_endpoint,
	ARG_OUT Nexus_Startpoint &	remote_comm_startpoint,
	ARG_IN int			timeout = -1) = 0;

    protected: virtual
    CIF_Bool
    establish_Connection(
	ARG_IN Nexus_Endpoint &		local_ctrl_endpoint,
	ARG_OUT Nexus_Endpoint &	local_comm_endpoint,
	ARG_OUT Nexus_Startpoint &	remote_comm_startpoint,
	ARG_IN int			timeout = -1) = 0;

    
    /**************************************************************************
			   Internal methods and data
    **************************************************************************/

    private:

    /* thread safety data structures */
    nexus_mutex_t			mutex;

    /* remote startpoint and local endpoint used to communication messages
       with the remote process */
    Nexus_Startpoint			startpoint;
    Nexus_Endpoint			endpoint;

    /* state of the connection */
    volatile enum Object_State
    {
	OBJECT_STATE_INACTIVE,
	OBJECT_STATE_ACTIVE,
	OBJECT_STATE_DELETED
    }					object_state;
};


#endif /* CIF_INCLUDE_CIF_CONNECTION_NEXUS */
