/******************************************************************************
Nexus_Receive_Buffer.C

Description:

  Classes to provide simple Nexus receive buffer functionality

CVS Information:

  $Source$
  $Date$
  $Revision$
  $State$
  $Author$
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "Nexus_internal.H"
#include "Nexus_Monitor.H"


/******************************************************************************
			 Nexus_Send_Buffer API methods
******************************************************************************/


Nexus_Receive_Buffer::Nexus_Receive_Buffer()
{
    object_state = OBJECT_STATE_INACTIVE;    
}


Nexus_Bool
Nexus_Receive_Buffer::activate(
    ARG_INOUT nexus_buffer_t &		nx_buffer)
{
    buffer = nx_buffer;

    object_state = OBJECT_STATE_ACTIVE;

    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}


Nexus_Bool
Nexus_Receive_Buffer::get_Byte(
    ARG_OUT Nexus_Byte *		array,
    ARG_IN unsigned			count)
{
    nexus_get_byte(&buffer, array, count);
    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}


Nexus_Bool
Nexus_Receive_Buffer::get_Char(
    ARG_OUT char *			array,
    ARG_IN unsigned			count)
{
    nexus_get_char(&buffer, array, count);
    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}


Nexus_Bool
Nexus_Receive_Buffer::get_Int(
    ARG_OUT int *			array,
    ARG_IN unsigned			count)
{
    nexus_get_int(&buffer, array, count);
    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}


Nexus_Bool
Nexus_Receive_Buffer::get_Startpoint(
    ARG_OUT Nexus_Startpoint *		startpoint,
    ARG_IN unsigned			count)
{
    int					i;
    
    for (i = 0; i < count; i++)
    {
	startpoint[i].get_Internal_Startpoint(&buffer);
    }

    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}

Nexus_Bool
Nexus_Receive_Buffer::deactivate()
{
    nexus_buffer_destroy(&buffer);

    object_state = OBJECT_STATE_ACTIVE;

    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}

Nexus_Receive_Buffer::~Nexus_Receive_Buffer()
{
    object_state = OBJECT_STATE_DELETED;
}

